clear
clc
%define Weight
Wn = 0.001;
Wu = 10*tf([1 10],[1 1000]);
Wdist = tf(0.25,[1 0.25],'outputname','f2');
Wp = 80*tf(0.1,[1 0.1]);
%define System Parameters
m1 = 1;
m2 = 2;
k2 = 1;
b1 = 0.05;
b2 = 0.05;
% this System Parameter is Real Uncertainty
k1 = ureal('k1',2,'Range',[1.2 2.8]);
%define Nominal system
a1c = [0 0 -1/m1  1/m2]'*k1;
a2c = [0 0  1/m1 -1/m2]'*k1 + [0 0 0 -k2/m2]';
a3c = [1 0 -b1/m1 b1/m2]';
a4c = [0 1 b1/m1 -(b1+b2)/m2]';
A  = [a1c a2c a3c a4c];
plant = ss(A,[0 0;0 0;1/m1 0;0 1/m2],[0 1 0 0],[0 0]);
plant.StateName = {'x1';'x2';'x1dot';'x2dot'};
plant.OutputName = {'x2'};
%define Uncertainty system
Wunmod = 2.6*tf([1 0],[1 40]);
Delta = ultidyn('Delta',[1 1]);
plant = plant * append(1+Delta*Wunmod,1);
plant.InputName = {'f1 (control)','f2 (disturbance)'};
%Create Standard configuration
systemnames = 'plant Wu Wp Wn Wdist';
inputvar = '[dist; noise; f1]';
outputvar = '[Wp; Wu; plant+Wn]';
input_to_Wdist = '[dist]';
input_to_plant = '[f1; Wdist]';
input_to_Wu = '[f1]';
input_to_Wp = '[plant]';
input_to_Wn = '[noise]';
cleanupsysic = 'yes';
IC = sysic;
opt = dksynOptions('NumberofAutoIterations',4,'DisplayWhileAutoIter','on');
[Kc,gc,mu_c,infoc] = dksyn(IC,1,1,opt);

% Mixed uncertainty
opt = dksynOptions('MixedMU','on','AutoScalingOrder',[5 6], ...
    'NumberofAutoIterations',4,'DisplayWhileAutoIter','on');
[Km,gm,mu_m,infom] = dksyn(IC,1,1,opt);

k1 = ureal('k1',2,'Range',[1.2 2.8]);
k1c = ucomplex('k1c',2,'Radius',0.8);  % complex approximation
% Plot 80 samples of the real and complex parameters
k1samp = usample(k1,80);
k1csamp = usample(k1c,80);
clf
figure(1)
plot(k1samp(:),0*k1samp(:),'rx',real(k1csamp(:)),imag(k1csamp(:)),'b*')
hold on
%Draw a Circle around k1
the=0:0.02*pi:2*pi;
z=sin(the)+cos(the)*i;
plot(real(0.8*z+2),imag(0.8*z),'b')
hold off
% Plot formatting
axis([1 3 -1 1]), axis square
ylabel('Imaginary'), xlabel('Real')
title('Real vs. complex uncertainty model for k1')

systemnames = 'plant Wdist';
inputvar = '[dist; f1]';
outputvar = '[Wdist; f1; plant; plant]';
input_to_Wdist = '[dist]';
input_to_plant = '[f1; Wdist]';
cleanupsysic = 'yes';
ICSIM = sysic;
 
clsimKm = lft(ICSIM,Km);
clsimKc = lft(ICSIM,Kc);
t = 0:.05:100;
dist = randn(size(t));
yKc = lsim(clsimKc.Nominal,dist,t);
yKm = lsim(clsimKm.Nominal,dist,t);
figure(2)
subplot(311)
plot(t,yKc(:,3),t,yKm(:,3),'-.r')
title('Nominal Disturbance Rejection Response')
ylabel('z2')
legend('Kc','Km')
subplot(312)
plot(t,yKc(:,2),':',t,yKm(:,2),':r')
ylabel('f1 (control)')
subplot(313)
plot(t,yKc(:,1),'k')
ylabel('f2 (disturbance)')
xlabel('Time (sec)')

clpKc = lft(IC,Kc);
clpKm = lft(IC,Km);
om = logspace(-1,1.4,80);
clpKcg = ufrd(clpKc,om);
clpKmg = ufrd(clpKm,om);
wopt = wcgopt('Sensitivity','off','FreqPtWise',0);
[maxgainKc,badpertKc] = wcgain(clpKcg,wopt);
[maxgainKm,badpertKm] = wcgain(clpKmg,wopt);

clsimKc_wc = usubs(clsimKc,badpertKc);
clsimKm_wc = usubs(clsimKm,badpertKm);
yKc_wc = lsim(clsimKc_wc,dist,t);
yKm_wc = lsim(clsimKm_wc,dist,t);

figure(3)
subplot(211)
plot(t,yKc_wc(:,3),'-.',t,yKm_wc(:,3))
title('Worst-Case Disturbance Rejection Response')
legend('Kc','Km')
ylabel('z2')
subplot(212)
plot(t,yKc_wc(:,2),':',t,yKm_wc(:,2),':')
ylabel('f1 (control)')

