clear
clc
gamma = ureal('gamma',2,'Perc',30);  % uncertain gain
tau = ureal('tau',1,'Perc',30);      % uncertain time-constant
wn = 50;
xi = 0.25;
P = tf(gamma,[tau 1]) * tf(wn^2,[1 2*xi*wn wn^2]);
% Add unmodeled dynamics
delta = ultidyn('delta',[1 1],'SampleStateDim',5,'Bound',1);
W = makeweight(0.1,20,10);
P = P * (1+W*delta);
Parray = usample(P,15);
Pn = P.NominalValue;
orderWt = 2;
omega=logspace(-3,3,100);
Parrayg = frd(Parray,omega);
[Usys,Info] = ucover(Parrayg,Pn,orderWt,'InputMult');
W1=zpk(Info.W1);
Wperf = makeweight(500,0.4,0.5);
Wnoise = 0.0025 * tf([25 7 1],[2.5e-5 .007 1]);
[ny,nu] = size(Usys);
M = iconnect;
d = icsignal(ny);
n = icsignal(ny);
u = icsignal(nu);
y = icsignal(ny);
M.Input = [d;n;u];
M.Output = [y;-(y+Wnoise*n)];
M.Equation{1} = equate(y,Usys*u+Wperf*d);
[npy,npu] = size(P);
MP = iconnect;
d = icsignal(npy);
n = icsignal(npy);
u = icsignal(npu);
y = icsignal(npy);
MP.Input = [d;n;u];
MP.Output = [y;-(y+Wnoise*n)];
MP.Equation{1} = equate(y,P*u+Wperf*d);
opt = dksynOptions('NumberofAutoIterations',5,'DisplayWhileAutoIter','on');
[Km,Clm,mbnm,infom]  = dksyn(M.System,ny,nu,opt);
[Kp,Clp,mbndp,infop] = dksyn(MP.System,npy,npu,opt);
figure(1)
clf
semilogx(infom{2}.MussvBnds(:,1),'b',infom{3}.MussvBnds(:,1),'k-.',...
infom{5}.MussvBnds(:,1),'r--')
legend('\mu_2','\mu_3', '\mu_5')
xlabel('Frequency (rad/sec)')
title('Structured singular values for Usys uncertain model')
 
figure(2)
clf
semilogx(infop{2}.MussvBnds(:,1),'b',infop{3}.MussvBnds(:,1),'k-.',...
infop{5}.MussvBnds(:,1),'r--')
legend('\mu_2','\mu_3', '\mu_5')
xlabel('Frequency (rad/sec)')
title('Structured singular values for P uncertain model')
